@@ -1,5 +1,6 @@
 #!/usr/bin/perl -w
 
+use lib '../blib/lib';
 use Getopt::Long;
 use POE qw(Component::IRC::State Wheel::ReadLine);
 use Data::Dumper;
@@ -28,6 +29,8 @@ print "$nick $server\n";
 
 my $irc = POE::Component::IRC::State->spawn( Nick => $nick, Server => $server, Port => $port, Ircname => $ircname, Username => $user, Password => $password );
 
+print STDOUT $irc->version(), "\n";
+
 POE::Session->create(
 	package_states => [
 		'main' => [ qw(_start _stop got_input parse_input _default irc_public) ],
@@ -100,7 +103,12 @@ sub parse_input {
 	  last SWITCH;
 	}
 	if ( $cmd eq 'dump_state' ) {
-    	  $heap->{readline_wheel}->put($_) for split /\n/, Dumper($irc->{STATE});
+	  unless (@args) {
+    	    $heap->{readline_wheel}->put($_) for split /\n/, Dumper($irc->{STATE});
+	  } else {
+	    open my $fh, ">", $args[0] or return;
+	    print $fh Dumper($irc->{STATE});
+	  }
 	  last SWITCH;
 	}
 	$irc->yield( $cmd => @args );
